SystemOrganization addCategory: #'Pier-FuelPersistence-Core'!

PRDelayedPersistency subclass: #PRFuelPersistency
	instanceVariableNames: 'directory directoryName namePrefix maxImages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FuelPersistence-Core'!

PRFuelPersistency class
	instanceVariableNames: 'defaultDirectoryName'!

PRFuelPersistency class
	instanceVariableNames: 'defaultDirectoryName'!

!PRFuelPersistency class methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
defaultDirectoryName
	defaultDirectoryName isNil ifTrue: [ defaultDirectoryName := 'snapshot' ].
	^ defaultDirectoryName! !

!PRFuelPersistency class methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
defaultDirectoryName: anObject
	defaultDirectoryName := anObject! !

!PRFuelPersistency class methodsFor: 'accessing-descriptions' stamp: 'YanniChiu 5/31/2011 23:38'!
descriptionDirectoryName
	^ MAStringDescription new
		comment: 'The directory path for snapshot files.';
		label: 'Snapshot Directory Name';
		accessor: #directoryName;
		default: nil;
		priority: 120;
		yourself! !

!PRFuelPersistency class methodsFor: 'accessing-descriptions' stamp: 'YanniChiu 5/31/2011 23:38'!
descriptionMaxImages
	^ MANumberDescription new
		comment: 'The maximum number of backup copies to keep';
		accessor: #maxImages;
		label: 'Backup Images';
		priority: 140;
		beRequired;
		default: 3;
		beInteger;
		yourself! !

!PRFuelPersistency class methodsFor: 'accessing-descriptions' stamp: 'YanniChiu 5/31/2011 23:38'!
descriptionNamePrefix
	^ MAStringDescription new
		comment: 'The prefix for snapshot file names.';
		label: 'Snapshot Prefix';
		accessor: #namePrefix;
		default: nil;
		priority: 130;
		yourself! !

!PRFuelPersistency class methodsFor: 'importing' stamp: 'YanniChiu 5/31/2011 23:43'!
importFromDirectoryReference: reference withPrefix: aString
	| persistency kernel |
	persistency := PRFuelPersistency new directory: reference; namePrefix: aString; yourself.
	kernel := persistency deserialize.
	^kernel
! !

!PRFuelPersistency class methodsFor: 'importing' stamp: 'YanniChiu 5/31/2011 23:38'!
importFromPathString: aPathString withPrefix: aString
	| directory |
	directory := (FSDiskFilesystem current resolveString: aPathString) asReference.
	^ self importFromDirectoryReference: directory withPrefix: aString
! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
cleanupBackupImages
	| entries |
	maxImages isNil ifTrue: [ ^ self ].
	entries := self directory entries select: [ :each | each isFile and: [ each basename beginsWith: self namePrefix ] ].
	entries size <= maxImages ifTrue: [ ^ self ].
	entries := (SortedCollection sortBlock: [ :a :b | a basename < b basename ]) addAll: entries; yourself.
	entries := entries allButLast: maxImages.
	entries do: [ :each | each reference delete ]! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
desanitize
	super desanitize.
	self directory: nil.
! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 6/13/2011 02:27'!
deserialize
	| ref stream |
	ref := self newestSnapshotIn: self directory.
	stream := MultiByteBinaryOrTextStream with: ref readStream contents.
	stream binary.
	stream converter: TextConverter default.
	stream reset.
	^ (FLMaterializer on: stream) materialize; root! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 6/13/2011 00:21'!
directory
	directory isNil ifTrue: [
		directory := FSFilesystem onDisk referenceTo: self directoryName.
		FSFilesystem onDisk ensureDirectory: directory.
	].
	^ directory! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
directory: anObject
	directory := anObject! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
directoryName
	"Answer the directoryName value. Use the default value if nil, but do not update the inst. var. to the default value;
	otherwise, it will be persisted, and thus prevent changes during deployment."
	^ (directoryName isNil or: [ directoryName isEmpty ])
		ifTrue: [ self class defaultDirectoryName ]
		ifFalse: [ directoryName ]! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
directoryName: anObject
	directoryName := anObject! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
execute: aCommand
	self mutex critical: [
		aCommand doExecuteDataStore.
		self apply: aCommand.
		aCommand isLogged ifTrue: [
			self add: aCommand context.
			self log: aCommand context.
			self start ] ]! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
exportToDisk: aPath
	self exportToSixxWriteStream: (SixxWriteStream newFileNamed: aPath)! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 6/13/2011 01:51'!
exportToFilesystemReference: reference
	self exportToFuelWriteStream: (FLSerializer on: reference writeStream)! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 6/13/2011 02:21'!
exportToFuelWriteStream: sws
	"[ [ sws serialize: self kernel ] ensure: [ sws close ] ] on: Exception do: [ :error | nil ]"
	sws serialize: self kernel! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
log: aContext
	| ref stream |
	ref := self directory / 'transactions.txt'.
	stream := ref writeStream.
	[
		stream position: ref entry size + 1.
		self log: aContext on: stream
	]
	ensure: [ stream close ]! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
maxImages
	^ maxImages! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
maxImages: anObject
	maxImages := anObject! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
namePrefix
	"Answer the namePrefix value. Use the default value if nil, but do not update the inst. var. to the default value;
	otherwise, it will be persisted, and thus prevent changes during deployment."
	^ (namePrefix isNil or: [ namePrefix isEmpty ])
		ifTrue: [ self kernel name, '_' ]
		ifFalse: [ namePrefix ]! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
namePrefix: aString
	namePrefix := aString! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
newestSnapshotIn: directoryReference
	| entries |
	entries := directoryReference entries select: [ :each | each isFile and: [ each basename beginsWith: self namePrefix ] ].
	entries := (SortedCollection sortBlock: [ :a :b | a basename >= b basename ]) addAll: entries; yourself.
	entries size > 0
		ifTrue: [ ^ entries first reference ]
		ifFalse: [ self error: 'No snapshots found in: ', self directory greaseString ]! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
serialize: fileNameString
	self sanitize.
	self exportToFilesystemReference: self directory / fileNameString.
	self desanitize.
! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
snapshot
	| snapshotName |
	self snapshotWithName: (snapshotName := self snapshotName).
	self message: 'Last snapshot ' , snapshotName printString , ' at ' , GRPlatform current magritteTimeStampClass now greaseString.
	self cleanupBackupImages! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
snapshotName
	^ self snapshotNameFromDateAndTime: DateAndTime now! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
snapshotNameFromDateAndTime: dateAndTime
	^ self namePrefix , dateAndTime asSortableString , '.xml'! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
snapshotWithName: filename
	[ self serialize: filename ] valueUnpreemptively! !

!FSWriteStream methodsFor: '*pier-fuelpersistence-core' stamp: 'YanniChiu 6/13/2011 02:24'!
isBinary
	^ true! !
